#ifndef __rp3d_collisionworld__
#define __rp3d_collisionworld__

namespace rp3d

extern "C"

' ##################
' # CollisionWorld #
' ##################
' This class represent a world where it is possible to move bodies
' by hand and to test collision between each other.
' In this kind of world, the bodies movement is not computed using the laws of physics.

' CollisionWorld constructor/destructor
declare function CreateCollisionWorld() as CollisionWorld ptr
declare sub      DestroyCollisionWorld(byref cw as CollisionWorld ptr)

'CollisionBody constructors/destructor
declare function CollisionWorldCreateCollisionBodyFromPosRotXYZ(byval cw as CollisionWorld ptr, _
                                                                byval px as decimal, byval py as decimal, byval pz as decimal, _
                                                                byval rx as decimal, byval ry as decimal, byval rz as decimal) as CollisionBody ptr
declare function CollisionWorldCreateCollisionBodyFromPosRot(byval cw as CollisionWorld ptr, _
                                                             byref position as const Vector3, _
                                                             byref rotation as const Vector3) as CollisionBody ptr
declare function CollisionWorldCreateCollisionBody(byval cw as CollisionWorld ptr, _
                                                   byref t as const Transform) as CollisionBody ptr
declare sub      CollisionWorldDestroyCollisionBody(byval cw as CollisionWorld ptr, _
                                                    byref cb as CollisionBody ptr)

' methods

' reports line collisions via RaycastCallback
declare sub      CollisionWorldLinecast(byval cw as CollisionWorld ptr, _
                                        byref startPoint as const Vector3, _
                                        byref endPoint   as const Vector3, _
                                        byval categoryMaskBits as ushort=&HFFFF)
' reports ray collisions via RaycastCallback
declare sub      CollisionWorldRaycast(byval cw as CollisionWorld ptr, _
                                       byref ray as Ray, _
                                       byval categoryMaskBits as ushort=&HFFFF)
' Test if the AABBs of two bodies overlap
declare function CollisionWorldTestBodyAABBOverlap(byval cw as CollisionWorld ptr, _
                                                   byval body1 as const CollisionBody ptr, _
                                                   byval body2 as const CollisionBody ptr) as boolean
' Report via OverlapCallback all the bodies which have an AABB that overlaps with the AABB in parameter
declare sub      CollisionWorldTestAABBOverlap(byval cw as CollisionWorld ptr, _
                                               byref aabb as const AABB, _
                                               byval categoryMaskBits as ushort=&HFFFF)
' Return true if two bodies overlap
declare function CollisionWorldTestBodyOverlap(byval cw as CollisionWorld ptr, _
                                               byval body1 as CollisionBody ptr, _
                                               byval body2 as CollisionBody ptr) as boolean
' Report via OverlapCallback all the bodies that overlap with the body in parameter
declare sub      CollisionWorldTestOverlap(byval cw as CollisionWorld ptr, _
                                           byval body as CollisionBody ptr, _
                                           byval categoryMaskBits as ushort=&HFFFF)
' Test and report collisions between two bodies
declare sub      CollisionWorldTestBodyCollision(byval cw as CollisionWorld ptr, _
                                                 byval body1 as CollisionBody ptr, _
                                                 byval body2 as CollisionBody ptr)
' Test and report via CollisionCallback collisions between a body and all the others bodies of the world
declare sub      CollisionWorldTestCollision(byval cw as CollisionWorld ptr, _
                                             byval body as CollisionBody ptr, _
                                             byval categoryMaskBits as ushort=&HFFFF)
' Test and report via CollisionCallback collisions between all shapes of the world
declare sub      CollisionWorldTestAllCollision(byval cw as CollisionWorld ptr)

' Return the current world-space AABB of given proxy shape
declare function CollisionWorldGetWorldAABB(byval cw as CollisionWorld ptr, byval proxy as const ProxyShape ptr) as AABB

end extern 

end namespace

#endif '__rp3d_collisionworld__
