#ifndef __rp3d_concavemeshshape__
#define __rp3d_concavemeshshape__

namespace rp3d

' eVertexDataType and eIndexDataType are defined in rp3d_convexmeshshape.bi

enum eNormalDataType
  NORMAL_FLOAT_TYPE = 0
  NORMAL_DOUBLE_TYPE
end enum

enum eHeightDataType
  HEIGHT_FLOAT_TYPE = 0
  HEIGHT_DOUBLE_TYPE
  HEIGHT_INT_TYPE ' 32-bit
end enum 

type tTriangleVertexArray as any
type tTriangleMesh as any

'type tConcaveMesh extends Mesh
'  as tTriangleVertexArray ptr TriVertArray
'  as tTriangleMesh ptr        TriMesh         ' TriMesh.addSubpart(TriVertArray)
'  as ConcaveMeshShape ptr    MeshShape 
'end type

' #########################################
' # ConcaveShape extends CollisionShape   #
' # ConcaveMeshShape extends ConcaveShape #
' # HeightFieldShape extends ConcaveShape #
' #########################################

extern "C"

' #######################
' # TriangleVertexArray #
' #######################
' constructor
declare function CreateTriangleVertexArray(byval nbVertices as ulong, _
                                           byval verticesStart as const any ptr, _
                                           byval verticesStride as ulong, _
                                           byval nbTriangles as ulong, _
                                           byval indexesStart as const any ptr, _
                                           byval indexesStride as ulong, _
                                           byval vertexDataType as eVertexDataType, _
                                           byval indexDataType as eIndexDataType) as tTriangleVertexArray ptr

' constructor with normals
declare function CreateTriangleVertexNormalArray(byval bVertices as ulong, _
                                                 byval verticesStart as const any ptr, _
                                                 byval verticesStride as ulong, _
                                                 byval verticesNormalsStart as const any ptr, _
                                                 byval uverticesNormalsStride as ulong, _
                                                 byval nbTriangles as ulong, _
                                                 byval indexesStart as const any ptr, _
                                                 byval indexesStride as ulong, _
                                                 byval vertexDataType as eVertexDataType, _
                                                 byval normalDataType as eNormalDataType, _
                                                 byval indexDataType as eIndexDataType) as tTriangleVertexArray ptr

declare sub      DestroyTriangleVertexArray(byref tva as tTriangleVertexArray ptr)
' methods

' Return the vertex data type
declare function TriangleVertexArrayGetVertexDataType(byval tva as tTriangleVertexArray ptr) as eVertexDataType 
' Return the vertex normal data type
declare function TriangleVertexArrayGetVertexNormalDataType(byval tva as tTriangleVertexArray ptr) as eNormalDataType
' Return the index data type
declare function TriangleVertexArrayGetIndexDataType(byval tva as tTriangleVertexArray ptr) as eIndexDataType
' Return the number of vertices
declare function TriangleVertexArrayGetNbVertices(byval tva as tTriangleVertexArray ptr) as ulong
' Return the number of triangles
declare function TriangleVertexArrayGetNbTriangles(byval tva as tTriangleVertexArray ptr) as ulong
' Return the vertices stride (number of bytes)
declare function TriangleVertexArrayGetVerticesStride(byval tva as tTriangleVertexArray ptr) as ulong
' Return the vertex normals stride (number of bytes)
declare function TriangleVertexArrayGetVerticesNormalsStride(byval tva as tTriangleVertexArray ptr) as ulong
' Return the indices stride (number of bytes)
declare function TriangleVertexArrayGetIndicesStride(byval tva as tTriangleVertexArray ptr) as ulong
' Return the pointer to the start of the vertices array
declare function TriangleVertexArrayGetVerticesStart(byval tva as tTriangleVertexArray ptr) as const any ptr
' Return the pointer to the start of the vertex normals array
declare function TriangleVertexArrayGetVerticesNormalsStart(byval tva as tTriangleVertexArray ptr) as const any ptr
' Return the pointer to the start of the indices array
declare function TriangleVertexArrayGetIndicesStart(byval tva as tTriangleVertexArray ptr) as const any ptr
' Return the vertices coordinates of a triangle
declare sub      TriangleVertexArrayGetTriangleVertices(byval tva as tTriangleVertexArray ptr, _
                                                        byval triangleIndex as ulong, _
                                                        byval outTriangleVertices as Vector3 ptr)
' Return the three vertices normals of a triangle
declare sub      TriangleVertexArrayGetTriangleVerticesNormals(byval tva as tTriangleVertexArray ptr, _
                                                               byval triangleIndex as ulong, _
                                                               byval outTriangleVerticesNormals as Vector3 ptr)
' Return the indices of the three vertices of a given triangle in the array
declare sub      TriangleVertexArrayGetTriangleVerticesIndices(byval tva as tTriangleVertexArray ptr, _
                                                               byval triangleIndex as ulong, _
                                                               byval outVerticesIndices as ulong ptr)
' Return a vertex of the array
declare sub      TriangleVertexArrayGetVertex(byval tva as tTriangleVertexArray ptr, _
                                              byval vertexIndex as ulong, _
                                              byval outVertex as Vector3 ptr)
' Return a vertex normal of the array
declare sub      TriangleVertexArrayGetNormal(byval tva as tTriangleVertexArray ptr, _
                                              byval vertexIndex as ulong, _
                                              byval outNormal as Vector3 ptr)

' ################
' # TriangleMesh #
' ################
declare function CreateTriangleMesh() as tTriangleMesh ptr
declare sub      DestroyTriangleMesh(byref tm as tTriangleMesh ptr)

declare sub      TriangleMeshAddSubpart(byval tn as tTriangleMesh ptr, _
                                        byval tva as tTriangleVertexArray ptr)

declare function TriangleMeshGetNbSubparts(byval tm as tTriangleMesh ptr) as ulong
declare function TriangleMeshGetSubpart(byval tm as tTriangleMesh ptr, _
                                        byval indexSubpart as ulong) as tTriangleVertexArray ptr


' #########################################
' # ConcaveShape extends CollisionShape   #
' # ConcaveMeshShape extends ConcaveShape #
' #########################################
' This class represents a static concave mesh shape.
' Note that collision detection with a concave mesh shape can be very expensive.
' You should only use this shape for a static mesh.
declare function CreateConcaveMeshShape(byval tm as tTriangleMesh ptr) as ConcaveMeshShape ptr
declare function CreateScaledConcaveMeshShape(byval tm as tTriangleMesh ptr, _
                                              byref scale as Vector3) as ConcaveMeshShape ptr
declare function CreateScaledConcaveMeshShapeXYZ(byval tm as tTriangleMesh ptr, _
                                                 byval sx as decimal, _
                                                 byval sy as decimal, _
                                                 byval sz as decimal) as ConcaveMeshShape ptr

declare sub      DestroyConcaveMeshShape(byref cms as ConcaveMeshShape ptr)


' #########################################
' # ConcaveShape extends CollisionShape   #
' # ConcaveMeshShape extends ConcaveShape #
' # HeightFieldShape extends ConcaveShape #
' #########################################
' This class represents a static height field that can be used to represent a terrain.
' The height field is made of a grid with rows and columns with a height value at each grid point.
declare function CreateHeightFieldShape(byval nbGridColumns   as long, _
                                        byval nbGridRows      as long, _
                                        byval minHeight       as decimal, _
                                        byval maxHeight       as decimal, _
                                        byval heightFieldData as const any ptr, _
                                        byval dataType        as eHeightDataType) as HeightFieldShape ptr

declare sub      DestroyHeightFieldShape(byref hfs as HeightFieldShape ptr)


end extern

end namespace

#endif '__rp3d_concavemeshshape__
