#ifndef __rp3d_convexmeshshape__
#define __rp3d_convexmeshshape__

namespace rp3d

' #################################################
' # ConvexPolyhedronShape extends ConvexShape     #
' # ConvexMeshShape extends ConvexPolyhedronShape #
' #################################################
' This class represents a convex mesh shape.
' In order to create a convex mesh shape, you need to indicate the local-space position of the mesh vertices.
' The center of mass of the shape will be at the origin of the local-space geometry that you use to create the mesh.

enum eVertexDataType 
  VERTEX_FLOAT_TYPE = 0
  VERTEX_DOUBLE_TYPE
end enum  
enum eIndexDataType
  INDEX_INTEGER_TYPE = 0 ' 32-bit
  INDEX_SHORT_TYPE       ' 16-bit
end enum  

' ###############
' # PolygonFace #
' ###############
type PolygonFace
  as ulong nbVertices
  as ulong indexBase
end type

type PolygonVertexArray as any 
type PolyhedronMesh  as any

extern "C"

'declare function CreatePolygonFaces(byval nbFaces as ulong) as PolygonFace ptr

' ######################
' # PolygonVertexArray #
' ######################
' construtor
declare function CreatePolygonVertexArray(byval nbVertices     as ulong, _
                                          byval verticesStart  as const any ptr, _
                                          byval verticesStride as long, _
                                          byval indexesStart   as const any ptr, _
                                          byval indexesStride  as long, _
                                          byval nbFaces        as ulong, _
                                          byval facesStart     as PolygonFace ptr, _
                                          byval vertexDataType as eVertexDataType, _
                                          byval indexDataType  as eIndexDataType) as PolygonVertexArray ptr

declare sub      DestroyPolygonVertexArray(byref pva as PolygonVertexArray ptr)

' methods

' Return the vertex data type
declare function PolygonVertexArrayGetVertexDataType(byval pva as PolygonVertexArray ptr) as eVertexDataType
' Return the index data type
declare function PolygonVertexArrayGetIndexDataType(byval pva as PolygonVertexArray ptr) as eIndexDataType
' Return the number of vertices
declare function PolygonVertexArrayGetNbVertices(byval pva as PolygonVertexArray ptr) as ulong
' Return the number of faces
declare function PolygonVertexArrayGetNbFaces(byval pva as PolygonVertexArray ptr) as ulong
' Return the vertices stride (number of bytes)
declare function PolygonVertexArrayGetVerticesStride(byval pva as PolygonVertexArray ptr) as long
' Return the indices stride (number of bytes)
declare function PolygonVertexArrayGetIndicesStride(byval pva as PolygonVertexArray ptr) as long
' Return the vertex index of a given vertex in a face
declare function PolygonVertexArrayGetVertexIndexInFace(byval pva as PolygonVertexArray ptr, _
                                                        byval faceIndex as ulong, _
                                                        byval noVertexInFace as ulong) as ulong
' Return a polygon face of the polyhedron
declare function PolygonVertexArrayGetPolygonFace(byval pva as PolygonVertexArray ptr, _
                                                  byval faceIndex as ulong) as PolygonFace ptr
' Return the pointer to the start of the vertices array
declare function PolygonVertexArrayGetVerticesStart(byval pva as PolygonVertexArray ptr) as const any ptr
' Return the pointer to the start of the indices array
declare function PolygonVertexArrayGetIndicesStart(byval pva as PolygonVertexArray ptr) as const any ptr

'##################
'# PolyhedronMesh #
'##################
declare function CreatePolyhedronMesh(byval pva as PolygonVertexArray ptr) as PolyhedronMesh ptr
declare sub      DestroyPolyhedronMesh(byref phm as PolyhedronMesh ptr)

' #######################################################
' # ConvexShape           extends CollisionShape        #
' # ConvexPolyhedronShape extends ConvexShape           #
' # ConvexMeshShape       extends ConvexPolyhedronShape #
' #######################################################
' This class represents a convex mesh shape.
' In order to create a convex mesh shape, you need to indicate the local-space position of the mesh vertices.
' The center of mass of the shape will be at the origin of the local-space geometry that you use to create the mesh.
declare function CreateConvexMeshShape(byval phm as PolyhedronMesh ptr) as ConvexMeshShape ptr
declare function CreateScaledConvexMeshShape(byval phm as PolyhedronMesh ptr, _
                                             byref scale as Vector3) as ConvexMeshShape ptr
declare function CreateScaledConvexMeshShapeXYZ(byval phm as PolyhedronMesh ptr, _
                                                byval sx as decimal, _
                                                byval sy as decimal, _
                                                byval sz as decimal) as ConvexMeshShape ptr
declare sub      DestroyConvexMeshShape(byref cms as ConvexMeshShape ptr)

end extern

end namespace

#endif ' __rp3d_convexmeshshape__
