#ifndef __rp3d_dynamicsworld__
#define __rp3d_dynamicsworld__

namespace rp3d


' ########################################
' # DynamicsWorld extends CollisionWorld #
' ########################################
' This class represents a dynamics world.
' In a dynamics world, bodies can collide 
' and their movements are simulated using the laws of physics.

extern "C"

' constructors
declare function CreateDynamicsWorldXYZ(byval gx as decimal=0.0, _
                                        byval gy as decimal=-9.81, _
                                        byval gz as decimal=0.0) as DynamicsWorld ptr
declare function CreateDynamicsWorld(byref gravity as const Vector3 = type<Vector3>(0,-9.81,0)) as DynamicsWorld ptr
declare sub      DestroyDynamicsWorld(byref dw as DynamicsWorld ptr)
' methods
declare sub      DynamicsWorldUpdate(byval dw as DynamicsWorld ptr, byval timeStep as decimal)

declare function DynamicsWorldGetNbIterationsVelocitySolver(byval dw as DynamicsWorld ptr) as ulong
declare sub      DynamicsWorldSetNbIterationsVelocitySolver(byval dw as DynamicsWorld ptr, byval nbIterations as ulong)

declare function DynamicsWorldGetNbIterationsPositionSolver(byval dw as DynamicsWorld ptr) as ulong
declare sub      DynamicsWorldSetNbIterationsPositionSolver(byval dw as DynamicsWorld ptr, byval nbIterations as ulong)

declare function DynamicsWorldGetGravity(byval dw as DynamicsWorld ptr) as Vector3
declare sub      DynamicsWorldSetGravity(byval dw as DynamicsWorld ptr,byref gravity as Vector3)

declare function DynamicsWorldIsGravityEnabled(byval dw as DynamicsWorld ptr) as boolean
declare sub      DynamicsWorldSetIsGratityEnabled(byval dw as DynamicsWorld ptr,byval isGravityEnabled as boolean)

declare function DynamicsWorldIsSleepingEnabled(byval dw as DynamicsWorld ptr) as boolean
declare sub      DynamicsWorldEnableSleeping(byval dw as DynamicsWorld ptr, byval isSleepingEnabled as boolean)

declare function DynamicsWorldGetSleepLinearVelocity(byval dw as DynamicsWorld ptr) as decimal
declare sub      DynamicsWorldSetSleepLinearVelocity(byval dw as DynamicsWorld ptr, byval sleepLinearVelocity as decimal)

declare function DynamicsWorldGetSleepAngularVelocity(byval dw as DynamicsWorld ptr) as decimal
declare sub      DynamicsWorldSetSleepAngularVelocity(byval dw as DynamicsWorld ptr, byval sleepAngularVelocity as decimal)

declare function DynamicsWorldGetTimeBeforeSleep(byval dw as DynamicsWorld ptr) as decimal
declare sub      DynamicsWorldSetTimeBeforeSleep(byval dw as DynamicsWorld ptr, byval timeBeforeSleep as decimal)


' RigidBody constructors
declare function DynamicsWorldCreateRigidBodyPosRotXYZ(byval dw as DynamicsWorld ptr, _
                                                       byval px as decimal, byval py as decimal, byval pz as decimal, _
                                                       byval rx as decimal, byval ry as decimal, byval rz as decimal) as RigidBody ptr

declare function DynamicsWorldCreateRigidBodyPosRot(byval dw as DynamicsWorld ptr, _
                                                    byref position as const Vector3, _
                                                    byref rotation as const Vector3) as RigidBody ptr

declare function DynamicsWorldCreateRigidBody(byval dw as DynamicsWorld ptr, _
                                              byref t as const Transform) as RigidBody ptr

declare sub DynamicsWorldDestroyRigidBody(byval dw as DynamicsWorld ptr, _
                                          byref rb as RigidBody ptr)

declare function DynamicsWorldGetNbRigidBodies(byval dw as DynamicsWorld ptr) as ulong

' joint constructors
declare function DynamicsWorldCreateJoint             (byval dw as DynamicsWorld ptr, _
                                                       byref ji as const JointInfo) as Joint ptr
declare function DynamicsWorldCreateFixedJoint        (byval dw as DynamicsWorld ptr, _
                                                       byref ji as const JointInfo) as FixedJoint ptr
declare function DynamicsWorldCreateBallAndSocketJoint(byval dw as DynamicsWorld ptr, _
                                                       byref ji as const JointInfo) as BallAndSocketJoint ptr
declare function DynamicsWorldCreateHingeJoint        (byval dw as DynamicsWorld ptr, _
                                                       byref ji as const JointInfo) as HingeJoint ptr
declare function DynamicsWorldCreateSliderJoint       (byval dw as DynamicsWorld ptr, _
                                                       byref ji as const JointInfo) as SliderJoint ptr

declare sub      DynamicsWorldDestroyJoint(byval dw as DynamicsWorld ptr, _
                                           byref jo as Joint ptr)

declare function DynamicsWorldGetNbJoints(byval dw as DynamicsWorld ptr) as ulong

end extern

end namespace


#endif ' __rp3d_dynamicsworld__
